package com.yunzhi.roundabout.controller;

import com.alibaba.fastjson.JSONObject;
import com.yunzhi.roundabout.common.*;
import com.yunzhi.roundabout.entity.TaPerson;
import com.yunzhi.roundabout.service.ITaPersonService;
import com.yunzhi.roundabout.vo.LoginParam;
import com.yunzhi.roundabout.vo.MPLogin;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.*;

import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

@Api(tags = "登入/登出")
@RestController
public class LoginController extends BaseController {

    @Value("${biz.admin.account}")
    String adminAccount;

    @Value("${biz.admin.password}")
    String adminPassword;

    @Value("${biz.wxapi}")
    String wxApi;

    @Autowired
    HttpUtils httpUtils;

    @Autowired
    ITaPersonService iTaPersonService;

    @PostMapping("/admin/login")
    @ApiOperation(value="后端登录", notes = "后端登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean login(@ApiParam("登录参数") @RequestBody LoginParam loginParam) throws Exception {
        if (null == loginParam) {
            return ResponseBean.error("账户或密码不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        String userName = loginParam.getUserName();
        String password = loginParam.getPassword();

        if (StringUtils.isEmpty(userName) || StringUtils.isEmpty(password)) {
            return ResponseBean.error("账户或密码不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (!adminAccount.equals(userName)) {
            return ResponseBean.error("账户或密码不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        String targetPassword = md5(md5(adminPassword) + adminAccount);

        if (!checkPassword(password, targetPassword, adminAccount)) {
            return ResponseBean.error("账户或密码不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // 生成 token
        Map<String, Object> tokenMap = new HashMap<String, Object>() {{
           put("userId", adminAccount);
        }};
        String token = JWTUtils.encode(tokenMap);

        Map<String, Object> res = new HashMap<String, Object>() {{
//            put("user", sysUser);
            put("token", token);
        }};
        return ResponseBean.success(res);
    }

    /**
     * 根据id查询对象
     * @param code  登录CODE
     */
    @RequestMapping(value="/wx/login",method= RequestMethod.POST)
    @ApiOperation(value="微信登录", notes = "微信登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taPersonGet(@ApiParam("对象ID") @RequestParam String code) throws Exception{
        TaPerson taPerson = null;
        if ("123".equals(code)) {
            taPerson = iTaPersonService.getById(code);
        } else {

            String wxresp = httpUtils.get(wxApi + "/mp/userinfo?code=" + code);
            if (StringUtils.isEmpty(wxresp)) {
                return ResponseBean.error("登录参数不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
            }

            JSONObject obj = JSONObject.parseObject(wxresp);
            if (ResponseBean.CODE_SUCCESS != (Integer) obj.get("code")) {
                return ResponseBean.error((String) obj.get("message"), ResponseBean.ERROR_ILLEGAL_PARAMS);
            }

            JSONObject params = obj.getJSONObject("data");

            String openid = (String) params.get("openid");
            if (StringUtils.isEmpty(openid)) {
                return ResponseBean.error("系统出错, 请退出重试", ResponseBean.ERROR_ILLEGAL_PARAMS);
            }

            String nickname = (String) params.get("nickname");
            String headimgurl = (String) params.get("headImgUrl");

            taPerson = iTaPersonService.getByOpenId(openid);

            // 更新人员信息
            if (StringUtils.isEmpty(taPerson.getName())) {
                taPerson.setName(nickname);
                taPerson.setAvatar(headimgurl);
                iTaPersonService.updateById(taPerson);
            }
        }

        // 生成 token
        TaPerson finalTaPerson = taPerson;
        Map<String, Object> tokenMap = new HashMap<String, Object>() {{
            put("userId", finalTaPerson.getPersonId());
        }};
        String token = JWTUtils.encode(tokenMap);

        Map<String, Object> res = new HashMap<String, Object>() {{
            put("user", finalTaPerson);
            put("token", token);
        }};

        return ResponseBean.success(res);
    }

    private boolean checkPassword(String src, String targ, String salt) {
        return md5((src+salt)).equals(targ);
    }

    private String md5(String src) {
        return DigestUtils.md5DigestAsHex(src.getBytes(StandardCharsets.UTF_8));
    }
}
