package com.yunzhi.roundabout.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.roundabout.common.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.roundabout.service.ITaPersonService;
import com.yunzhi.roundabout.entity.TaPerson;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
    * 用户表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-01-13
 */

@Api(tags = "用户表")
@RestController
@RequestMapping("/")
public class TaPersonController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaPersonController.class);

    @Autowired
    public ITaPersonService iTaPersonService;


    @Value("${biz.wxapi}")
    String wxApi;

    @Autowired
    HttpUtils httpUtils;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taPerson",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taPersonList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @ApiParam(value = "开始时间", example = "2021-01-01") @RequestParam(value ="startDate", required = false) String startDate,
                                     @ApiParam(value = "结束时间", example = "2021-10-31") @RequestParam(value ="endDate", required = false) String endDate) throws Exception{

        LocalDateTime start = null, end = null;

        if (!StringUtils.isEmpty(startDate)) {
            start = DateUtils.from(startDate + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(endDate)) {
            end = DateUtils.from(endDate + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        IPage<TaPerson> pg = new Page<>(pageNum, pageSize);
        IPage<TaPerson> result = iTaPersonService.getPageBy(pg, start, end);
        return ResponseBean.success(result);
    }

    @RequestMapping(value="/taPerson",method= RequestMethod.POST)
    @ApiOperation(value="导出Excel", notes = "导出Excel", httpMethod = "POST", response = ResponseBean.class)
    public void exportXls(@ApiParam(value = "开始时间", example = "2021-01-01") @RequestParam(value ="startDate", required = false) String startDate,
                          @ApiParam(value = "结束时间", example = "2021-10-31") @RequestParam(value ="endDate", required = false) String endDate,
                          HttpServletResponse response) throws Exception {
        LocalDateTime start = null, end = null;

        if (!StringUtils.isEmpty(startDate)) {
            start = DateUtils.from(startDate + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(endDate)) {
            end = DateUtils.from(endDate + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        List<TaPerson> result = iTaPersonService.getListBy(start, end);
        ExcelUtils.flush(response, TaPerson.class, result, "参与人员名单");
    }


    /**
     * 分页查询列表
     * @return
     */
    @RequestMapping(value="/wx/top",method= RequestMethod.GET)
    @ApiOperation(value="排行榜", notes = "排行榜", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean top() throws Exception{

        QueryWrapper<TaPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("total_millsec", 0);
        queryWrapper.orderByAsc("total_millsec");
        queryWrapper.orderByAsc("join_date");
        queryWrapper.last("limit 100");

        return ResponseBean.success(iTaPersonService.list(queryWrapper));
    }

    /**
     * wx-jdk
     */
    @RequestMapping(value="/wx/jssdk",method= RequestMethod.GET)
    @ApiOperation(value="WXJDK", notes = "WXJDK", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean gameStart(@ApiParam("分享地址") @RequestParam String url) throws Exception{

        String wxresp = httpUtils.get(wxApi + "/mp/jssdk?url=" + StringUtils.encodeURIComponent(url) );
        if (StringUtils.isEmpty(wxresp)) {
            return ResponseBean.error("分享地址不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        JSONObject obj = JSONObject.parseObject(wxresp);
        if (ResponseBean.CODE_SUCCESS != (Integer) obj.get("code")) {
            return ResponseBean.error((String) obj.get("message"), ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        JSONObject data = obj.getJSONObject("data");
        return ResponseBean.success(data);
    }


    /**
     * 游戏开始
     */
    @RequestMapping(value="/wx/game/start",method= RequestMethod.POST)
    @ApiOperation(value="游戏开始", notes = "游戏开始", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean gameStart(HttpServletRequest request) throws Exception{
        TaPerson taPerson = currentPerson(request);
        long startTime = System.currentTimeMillis();

        taPerson.setLastStart(String.valueOf(startTime));
        iTaPersonService.updateById(taPerson);
        return ResponseBean.success(null);
    }

    /**
     * 游戏开始
     */
    @RequestMapping(value="/wx/game/over",method= RequestMethod.POST)
    @ApiOperation(value="游戏结束", notes = "游戏结束", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean gameOver(HttpServletRequest request) throws Exception{
        TaPerson taPerson = currentPerson(request);
        Long startTime = Long.parseLong(taPerson.getLastStart());
        if (startTime == null) {
            throw new Exception("请刷新重试");
        }

        long endTime = System.currentTimeMillis();
        long totalMillsec = endTime - startTime;

        int oneSec = 1000;
        int oneHour = 60 * 60 * 1000;

        // 防止异常
        if (totalMillsec < oneSec || totalMillsec > oneHour) {
            throw new Exception("请刷新重试");
        }

        taPerson.setLastEnd(String.valueOf(endTime));

        // 判断是否之前玩过
        if (taPerson.getTotalMillsec() > 0) {
            // 如果本次时间比上次要少, 则更新本次时间
            if (totalMillsec < taPerson.getTotalMillsec()) {
                taPerson.setStartTime(String.valueOf(startTime));
                taPerson.setEndTime(String.valueOf(endTime));
                taPerson.setTotalMillsec(totalMillsec);
                taPerson.setJoinDate(LocalDateTime.now());
            } else {
                // 否则, 本次游戏不会被算入排行榜
            }
        } else {
            taPerson.setStartTime(String.valueOf(startTime));
            taPerson.setEndTime(String.valueOf(endTime));
            taPerson.setTotalMillsec(totalMillsec);
            taPerson.setJoinDate(LocalDateTime.now());
        }

        iTaPersonService.updateById(taPerson);
        return ResponseBean.success(null);
    }

//    /**
//     * 保存对象
//     * @param taPerson 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taPerson",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean taPersonAdd(@ApiParam("保存内容") @RequestBody TaPerson taPerson) throws Exception{
//
//        if (iTaPersonService.save(taPerson)){
//            return ResponseBean.success(taPerson);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taPerson/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taPersonDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
//        if(iTaPersonService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }

//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taPerson 实体对象
//     * @return
//     */
//    @RequestMapping(value="/wx/taPerson/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taPersonUpdate(@ApiParam("对象ID") @PathVariable String id,
//                                        @ApiParam("更新内容") @RequestBody TaPerson taPerson) throws Exception{
//
//        TaPerson origin = iTaPersonService.getByPhone(taPerson.getPhone());
//        if (null != origin && !id.equals(origin.getPersonId())) {
//            return ResponseBean.error("手机号已存在, 请重新填写", ResponseBean.ERROR_UNAVAILABLE);
//        }
//
//        origin = iTaPersonService.getById(id);
//        if (null == origin) {
//            return ResponseBean.error("人员信息校验失败, 请刷新重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//
//        taPerson.setPersonId(id);
//
//        if (iTaPersonService.updateName(taPerson)){
//            return ResponseBean.success(iTaPersonService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }

}
