package com.yunzhi.roundabout.interceptor;

import com.yunzhi.roundabout.common.JWTUtils;
import com.yunzhi.roundabout.common.ResponseBean;
import com.yunzhi.roundabout.common.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Map;


@Slf4j
@Component
public class PermissionInterceptor implements HandlerInterceptor {

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {
//        log.info("Request URI: {}", request.getRequestURI());

        // JWT
        String jws = JWTUtils.getToken(request);
        if (StringUtils.isEmpty(jws)) {
            responseTokenError(response, "鉴权失败, 请先进行登录操作");
            return false;
        }

        try {
            Map<String, Object> tokenMap = JWTUtils.decode(jws);
        } catch (Exception e) {
            log.error("鉴权失败: {}", e.getMessage());
            responseTokenError(response, "鉴权失败, 请重新登录");
            return false;
        }

        return true;
    }

    private void responseTokenError(HttpServletResponse response, String error) throws IOException {
        response.addHeader("Content-type", "application/json");
        response.getOutputStream().write(
                ResponseBean.error(error, ResponseBean.ERROR_AUTH_FAIL).toJSON().getBytes(StandardCharsets.UTF_8));
    }
}
